package gov.va.cpe.vpr.dao.jds;

import gov.va.cpe.vpr.dao.IVprSyncStatusDao;
import gov.va.cpe.vpr.pom.IGenericPOMObjectDAO;
import gov.va.cpe.vpr.pom.jds.JdsOperations;
import gov.va.cpe.vpr.pom.jds.JdsPOMObjectDAO;
import gov.va.cpe.vpr.sync.SyncStatus;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;

import java.util.List;

public class JdsVprSyncStatusDao extends JdsPOMObjectDAO<SyncStatus> implements IVprSyncStatusDao {

    public static final String SYNC_STATUS_PATIENT_INDEX = "status-pt";
    public static final String SYNC_STATUS_LOADING_INDEX = "status-loading";
    public static final String SYNC_STATUS_LOADED_INDEX = "status-loaded";

    public JdsVprSyncStatusDao(IGenericPOMObjectDAO genericDao, JdsOperations jdsTemplate) {
		super(SyncStatus.class, genericDao, jdsTemplate);
	}

	@Override
	public SyncStatus findOneByPid(String pid) {
        return getGenericDao().findOneByIndexAndRange(SyncStatus.class, SYNC_STATUS_PATIENT_INDEX, pid);
	}

    @Override
    public int countLoadingPatients() {
        return getGenericDao().count(SYNC_STATUS_LOADING_INDEX);
    }

    @Override
    public int countLoadedPatients() {
        return getGenericDao().count(SYNC_STATUS_LOADED_INDEX);
    }

    @Override
    public SyncStatus findOneForOperational() {
        return getGenericDao().findByUID(SyncStatus.class, SyncStatus.OPERATIONAL_DATA_STATUS_UID);
    }

    @Override
    public List<SyncStatus> findAllLoadingPatientStatii() {
        return getGenericDao().findAllByIndex(SyncStatus.class, SYNC_STATUS_LOADING_INDEX);
    }

    @Override
    public List<SyncStatus> findAllPatientStatii() {
        return getGenericDao().findAllByIndex(SyncStatus.class, SYNC_STATUS_PATIENT_INDEX);
    }

    @Override
    public Page<SyncStatus> findAllPatientStatii(Pageable pageable) {
        return getGenericDao().findAllByIndex(SyncStatus.class, SYNC_STATUS_PATIENT_INDEX, pageable);
    }
}
