package gov.va.cpe.vpr.pom;

import com.fasterxml.jackson.annotation.JsonIgnore;
import gov.va.cpe.vpr.PatientDemographics;
import gov.va.cpe.vpr.frameeng.IFrameEvent;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Created by Jim on 11/19/14.
 */
public class POMObjectEvent<T extends IPOMObject> extends IFrameEvent<T> {
    public static enum Type {
        CREATE, // new record, does not exist before
        UPDATE, // updated exiting record
        RELOAD, // user/system initalized load/reload
        EVAL // manually injected/evaluated event?
    }

    protected String uid;
    protected String pid;
    protected Type type;
    protected List<Change> changes;
    private transient PatientDemographics patient;

    public POMObjectEvent(T source) {
        this(source, Type.EVAL, null);
        if(source instanceof IPatientObject) {
            this.pid = ((IPatientObject)source).getPid();
        }
    }

    public POMObjectEvent(T source, Type type, List<Change> changes) {
        super(source);
        this.uid = source.getUid();
        if(source instanceof IPatientObject) {
            this.pid = ((IPatientObject)source).getPid();
        }
        this.type = type;
        this.changes = (changes == null) ? new ArrayList<Change>() : changes;
    }

    /** Simple constructor for building events w/o having a domain object */
    public POMObjectEvent(String uid, Type type) {
        super(null);
        this.uid = uid;
        if(source instanceof IPatientObject) {
            this.pid = ((IPatientObject)source).getPid();
        }
        this.type = type;
        this.changes = new ArrayList<Change>();
    }

    public static class Change implements Serializable {
        private static final long serialVersionUID = 1L;
        public String FIELD;
        public Serializable OLD_VALUE;
        public Serializable NEW_VALUE;

        public Change(String field, Serializable oldVal, Serializable newVal) {
            FIELD = field;
            OLD_VALUE = oldVal;
            NEW_VALUE = newVal;
        }

        @Override
        public String toString() {
            return FIELD + ": " + OLD_VALUE + " -> " + NEW_VALUE;
        }
    }

    public void reconsitute(IGenericPatientObjectDAO dao, IPatientDAO patdao) {
        if (this.source == null) {
            this.source = dao.findByUID(getSourceClass(), getUID());
        }
        if (this.patient == null && pid!=null) {
            this.patient = patdao.findByPid(pid);
        }
    }

    public String getPID() {
        return this.pid;
    }

    @JsonIgnore
    public PatientDemographics getPatient() {
        return this.patient;
    }

    public void setParams(Map<String, Object> params) {
        this.params = params;
    }

    public void setType(Type type) {
        this.type = type;
    }

    public List<Change> getChanges() {
        return (changes == null) ? new ArrayList<Change>() : changes;
    }

    public Type getType() {
        return type;
    }

    public String getUID() {
        return this.uid;
    }

    @Override
    @SuppressWarnings("unchecked")
    public T getSource() {
        return (T) super.getSource();
    }

    @Override
    public String toString() {
        return getClass().getName() + "<" + getSourceClass().getSimpleName() + ">: " + ((getType() == Type.CREATE) ? "CREATE" : getChanges());
    }
}
