package gov.va.cpe.vpr.pom.jds;

import gov.va.cpe.vpr.PatientDemographics;
import gov.va.cpe.vpr.PidUtils;
import gov.va.cpe.vpr.pom.IPatientDAO;
import gov.va.hmp.jsonc.JsonCCollection;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class JdsPatientDAO extends JdsPatientObjectDAO<PatientDemographics> implements IPatientDAO {

    @Override
    public int count() {
        JsonCCollection<Map<String, Object>> jsonC = jdsTemplate.getForJsonC("/vpr/all/count/patient");
        if (jsonC.getItems().size() == 0) {
            return 0;
        }
        Map<String, Object> topicCount = jsonC.getItems().get(0);
        return (Integer) topicCount.get("count");
    }

	public JdsPatientDAO() {
        super(PatientDemographics.class);
    }

    @Override
    public PatientDemographics findByIcn(String icn) {
        if (StringUtils.isEmpty(icn)) return null;
        return jdsTemplate.getForObject(PatientDemographics.class, "/vpr/pid/" + icn);
    }

    @Override
    public PatientDemographics findByPid(String pid) {
        if (StringUtils.isEmpty(pid)) return null;
        PatientDemographics pt = jdsTemplate.getForObject(PatientDemographics.class, "/vpr/pid/" + pid);
        if (!isLoaded(pt)) return null;
        return pt;
    }

    @Override
    public PatientDemographics findByLocalId(String systemOrFacilityCode, String localPatientId) {
        if (StringUtils.isEmpty(localPatientId)) return null;

        String pid = getQualifiedDfn(systemOrFacilityCode, localPatientId);
        PatientDemographics pt = jdsTemplate.getForObject(PatientDemographics.class, "/vpr/pid/" + pid);
        if (!isLoaded(pt)) return null;
        return pt;
    }

    private boolean isLoaded(PatientDemographics pt) {
        return (pt != null);
    }

    private String getQualifiedDfn(String vistaIdOrFacilityCode, String dfn) {
        return vistaIdOrFacilityCode + PidUtils.SEPARATOR + dfn;
    }

    @Override
    public Page<PatientDemographics> findAll(Pageable pageable) {
        return genericDao.findAll(PatientDemographics.class, pageable);
    }

    @Override
    public List<String> listLoadedPatientIds() {
        JsonCCollection<Map<String, Object>> jsonC = jdsTemplate.getForJsonC("/data/index/status-loaded");
        ArrayList<String> rslt = new ArrayList<String>();
        for (Map<String, Object> pt : jsonC.getItems()) {
            if (pt.containsKey("pid")) {
                String pid = pt.get("pid").toString();
                if (StringUtils.hasText(pid))
                    rslt.add(pid);
            }
        }
        return rslt;
    }

    @Override
    public Map<String, Object> getSynchedCollectionCounts(String pid) {
        Map<String, Object> rslt = new HashMap<>();
        JsonCCollection<Map<String, Object>> jsonc = jdsTemplate.getForJsonC("/vpr/"+pid+"/count/collection");
        if (jsonc == null) {
            return rslt;
        }
        for(Map<String, Object> row: jsonc.getItems()) {
            String key = row.get("topic").toString();
            Integer value = Integer.valueOf(row.get("count").toString());
            rslt.put(key, value);
        }
        return rslt;
    }

    @Override
    public PatientDemographics save(PatientDemographics pt) {
        return genericDao.save(pt);
    }
}
