package gov.va.cpe.vpr.queryeng;

import gov.va.cpe.vpr.Allergy;
import gov.va.cpe.vpr.Document;
import gov.va.cpe.vpr.PatientDemographics;
import gov.va.cpe.vpr.PatientRecordFlag;
import gov.va.cpe.vpr.frameeng.FrameJob;
import gov.va.cpe.vpr.frameeng.POMObjectEventTrigger;
import gov.va.cpe.vpr.pom.IPatientDAO;
import gov.va.cpe.vpr.pom.JSONViews;
import gov.va.cpe.vpr.pom.POMObjectEvent;
import gov.va.cpe.vpr.queryeng.query.JDSQuery;
import gov.va.cpe.vpr.queryeng.query.QueryDef;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * Created by Jim on 10/17/2014.
 */
@Component(value = "gov.va.cpe.vpr.queryeng.CWADFViewDef")
@Scope("prototype")
public class CWADFViewDef extends ViewDef {

    public POMObjectEventTrigger<Document> dtrig;
    public POMObjectEventTrigger<Allergy> atrig;
    public POMObjectEventTrigger<PatientDemographics> ftrig;

    public CWADFViewDef() {

        declareParam(new ViewParam.PatientIDParam());
        this.domainClasses.add("Document");
        this.domainClasses.add("Allergy");
        QueryDef querydef = new QueryDef("cwad");

        Query q1 = addQuery(new JDSQuery("uid", querydef) {
            protected void filterTransformResults(RenderTask task, Map<String, Object> params, List<Map<String, Object>> items) {
                String pid = task.getParamStr("pid");
                List<Map<String, Object>> flags = findFlags(pid, task);
                if(items!=null) {items.addAll(flags);}
                super.filterTransformResults(task, params, items);
            }
        });

        addQuery(q1);

        dtrig = this.addTrigger(new POMObjectEventTrigger<Document>(Document.class));
        atrig = this.addTrigger(new POMObjectEventTrigger<Allergy>(Allergy.class));
        ftrig = this.addTrigger(new POMObjectEventTrigger<PatientDemographics>(PatientDemographics.class));
    }

    @Override
    protected void execAlt(FrameJob.FrameTask task) {
        boolean broadcast = false;
        if (atrig.isTriggerOf(task) || ftrig.isTriggerOf(task)) {broadcast=true;}
        else if(dtrig.isTriggerOf(task)) {
            POMObjectEvent<Document> evt = dtrig.getEventOf(task);
            Document doc = evt.getSource();
            String c = doc.getDocumentTypeCode();
            String s = doc.getStatus();
            if((c.equals("C")||c.equals("D"))/*&&(s.equals("COMPLETED")||s.equals("AMENDED"))*/) { // Per Mel, 10/22/2014, docs can fall off of CWAD in some cases, so I will loosen the filter for events that might update CWAD.
                broadcast=true;
            }
        }
        if(broadcast) {
            super.execAlt(task);
        }
    }

    private List<Map<String, Object>> findFlags(String pid, RenderTask task) {
        IPatientDAO patientDao = task.getResource(IPatientDAO.class);
        PatientDemographics dems = patientDao.findByPid(pid);
        if (dems != null && dems.getPatientRecordFlag() != null) {
            int i = 0;
            List<Map<String, Object>> flagItems = new ArrayList<>();
            for (PatientRecordFlag flag : dems.getPatientRecordFlag()) {
                HashMap<String, Object> data = new HashMap<>(flag.getData(JSONViews.WSView.class));
                data.put("uid", dems.getUid() + ":flag:" + i++);
                flagItems.add(data);
            }
            return flagItems;
        }
        return Collections.emptyList();
    }
}
