package gov.va.cpe.vpr.sync;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import gov.va.cpe.vpr.PidUtils;
import gov.va.cpe.vpr.dao.IVprSyncStatusDao;
import gov.va.hmp.HmpProperties;
import gov.va.hmp.hub.VistaAccount;
import gov.va.hmp.hub.dao.IVistaAccountDao;
import gov.va.hmp.vista.rpc.RpcOperations;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.EnvironmentAware;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeSet;

import javax.jms.JMSException;

import static gov.va.cpe.vpr.sync.vista.SynchronizationRpcConstants.VPR_STREAM_API_RPC_URI;
import static gov.va.hmp.vista.util.RpcUriUtils.VISTA_RPC_BROKER_SCHEME;

/**
 * Created by Jim on 12/17/14.
 */
@Component("syncIntegrityCheck")
public class SyncIntegrityCheck implements Runnable, EnvironmentAware {

    private RpcOperations synchronizationRpcTemplate;
    private IVprSyncStatusDao syncStatusDao;
    private Environment environment;
    private IVistaAccountDao vistaAccountDao;
    private ISyncService syncService;

    @Autowired
    public void setSynchronizationRpcTemplate(RpcOperations synchronizationRpcTemplate) {
        this.synchronizationRpcTemplate = synchronizationRpcTemplate;
    }

    @Autowired
    public void setSyncStatusDao(IVprSyncStatusDao syncStatusDao) {
        this.syncStatusDao = syncStatusDao;
    }

    @Autowired
    public void setVistaAccountDao(IVistaAccountDao vistaAccountDao) {
        this.vistaAccountDao = vistaAccountDao;
    }

    @Autowired
    public void setSyncService(ISyncService syncService) {
        this.syncService = syncService;
    }

    @Override
    public void run() {
        long pq = -1;
        List<Map<String, Object>> pqueues = null;
        try {
            pqueues = syncService.getPatientQueueSizes();
            pq = 0;
            for (Map<String, Object> roe : pqueues) {
                pq = pq + (Long.valueOf(roe.get("size").toString()));
            }
        } catch (JMSException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    	if(pq<=0) {
    		// JC 12-18-2014 (got my private pilot license today, by the way) Need to skip MBean use for now, it isn't hooked up, might be because of MessagingConfig but not sure.
    		// As an alternative, I will use the same method we are using to count what's left in the patient statuses.
//        if(syncService.getImportQueueSize()<=0) {
            List<VistaAccount> accounts = vistaAccountDao.findAllByVistaIdIsNotNull();
            for (VistaAccount account : accounts) {
                if (!account.isVprAutoUpdate()) continue;
                checkPatientSubscriptionStatusForVistaSystemID(account.getVistaId());
            }
        }
    }

    private void checkPatientSubscriptionStatusForVistaSystemID(String vistaId) {
        Map<String, Object> params = new HashMap<>();
        String version = environment.getProperty(HmpProperties.VERSION);

        params.put("command", "getSyncStatii");
        params.put("hmpVersion", version);
        params.put("server", environment.getProperty(HmpProperties.SERVER_ID));
        String url = VISTA_RPC_BROKER_SCHEME + "://" + vistaId + VPR_STREAM_API_RPC_URI;

        JsonNode json = synchronizationRpcTemplate.executeForJson(url, params);

        JsonNode items = json.path("data").findValue("items");

        if (items != null && items.isArray()) {
            resolveSyncStatusDifferences((ArrayNode) items, vistaId);
        }
    }

    private void resolveSyncStatusDifferences(ArrayNode vistaStatii, String vistaId) {
        TreeSet<String> jdsCompleteDfns = new TreeSet<>();
        TreeSet<String> vistaCompleteDfns = new TreeSet<>();
        TreeSet<String> resubscribeDfns = new TreeSet<>();
        HashMap<String, SyncStatus> syncStatiiByDfn = new HashMap<>();
        List<SyncStatus> jdsStatii = syncStatusDao.findAllPatientStatii();


        for(SyncStatus stat: jdsStatii) {
            SyncStatus.VistaAccountSyncStatus vstat = stat.getVistaAccountSyncStatusForSystemId(vistaId);
            if(vstat!=null && StringUtils.hasText(vstat.getDfn()) && vstat.isSyncComplete()) {
                jdsCompleteDfns.add(vstat.getDfn());
            }
        }

        for(JsonNode stat: vistaStatii) {
            if(stat.get("status").asInt()==3) {
                vistaCompleteDfns.add(stat.get("pid").asText());
            }
        }

        for(String dfn: vistaCompleteDfns) {
            if(!jdsCompleteDfns.contains(dfn)) {
                resubscribeDfns.add(dfn);
            }
        }
        for(String dfn: jdsCompleteDfns) {
            if(!vistaCompleteDfns.contains(dfn)) {
                resubscribeDfns.add(dfn);
            }
        }
        for(String dfn: resubscribeDfns) {
        	System.out.println("******************** Clearing and resubscribing "+dfn);
            syncService.unsubscribePatient(vistaId, PidUtils.getPid(vistaId, dfn));
            syncService.subscribePatient(vistaId, PidUtils.getPid(vistaId, dfn));
        }
    }

    @Override
    public void setEnvironment(Environment environment) {
        this.environment = environment;
    }
}
