package gov.va.cpe.vpr.sync.msg;

import gov.va.cpe.vpr.PatientDemographics;
import gov.va.cpe.vpr.PatientFacility;
import gov.va.cpe.vpr.dao.ISolrDao;
import gov.va.cpe.vpr.dao.IVprSyncErrorDao;
import gov.va.cpe.vpr.dao.IVprSyncStatusDao;
import gov.va.cpe.vpr.pom.IPatientDAO;
import gov.va.cpe.vpr.sync.ISyncService;
import gov.va.cpe.vpr.sync.SyncError;
import gov.va.cpe.vpr.sync.SyncMessageConstants;
import gov.va.cpe.vpr.sync.SyncStatus;
import gov.va.cpe.vpr.sync.vista.IVistaOperationalDataDAO;
import gov.va.cpe.vpr.sync.vista.IVistaVprDataExtractEventStreamDAO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jms.listener.SessionAwareMessageListener;
import org.springframework.jms.support.converter.SimpleMessageConverter;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.Session;
import java.util.Map;

public class ClearPatientMessageHandler implements SessionAwareMessageListener {
    private static Logger log = LoggerFactory.getLogger(ClearPatientMessageHandler.class);

    @Autowired
    private IPatientDAO patientDao;

    @Autowired
    private ISolrDao solrDao;

    private IVprSyncStatusDao vprSyncStatusDao;

    @Autowired
    private IVistaVprDataExtractEventStreamDAO vistaPatientDataService;
    @Autowired

    private IVistaOperationalDataDAO vistaOperationalDataService;

    private IVprSyncErrorDao errorDao;

    @Autowired
    public void setErrorDao(IVprSyncErrorDao errorDao) {
        this.errorDao = errorDao;
    }

    @Autowired
    public void setVprSyncStatusDao(IVprSyncStatusDao vprSyncStatusDao) {
        this.vprSyncStatusDao = vprSyncStatusDao;
    }

    private static Logger LOGGER = LoggerFactory.getLogger(ReindexPatientMessageHandler.class);


    @Autowired
    SimpleMessageConverter converter;

    @Autowired
    private ISyncService syncService;

    public void onMessage(Message message, Session session) {
        Map msg = null;
        try {
            msg = (Map) converter.fromMessage(message);
            String pid = (String) msg.get(SyncMessageConstants.PATIENT_ID);
            Assert.hasText(pid, "[Assertion failed] - 'pid' must have text; it must not be null, empty, or blank");

            log.debug("Getting patient for PID " + pid + " via class " + patientDao.getClass().getName());
            PatientDemographics pt = patientDao.findByPid(pid);

            if (pt == null) return;

            log.debug("Clearing patient " + pt.getFamilyName() + " with UID " + pt.getUid() + " AND pid " + pt.getPid());

            deletePatient(pid);
            unsubscribePatient(pt);
        } catch (Exception e) {
            LOGGER.error("Error synching patient data: " + e.getMessage(), e);
            try {
                errorDao.save(new SyncError(message, msg, e));
                session.recover();
            } catch (JMSException e1) {
                LOGGER.error("Exception while saving sync error", e1);
            }
        }
    }

    private void deletePatient(final String pid) {
        patientDao.deleteByPID(pid);

        syncService.deleteErrorByPatientId(pid);

        solrDao.deleteByQuery("pid:" + pid);

        SyncStatus stat = vprSyncStatusDao.findOneByPid(pid);
        if(stat!=null) {
            vprSyncStatusDao.delete(stat);
        }
    }

    private void unsubscribePatient(PatientDemographics pt) {
        for (PatientFacility f : pt.getFacility()) {
            if (StringUtils.hasText(f.getSystemId())) {
                final String localPatientId = f.getLocalPatientId();
                if (!StringUtils.hasText(localPatientId)) {
                    continue;
                }

                try {
                    //vistaPatientDataService.unsubscribePatient(f.getSystemId(), localPatientId, serverId);
                    vistaPatientDataService.unsubscribePatient(f.getSystemId(), pt);
                    log.debug("unsubscribed from VPR updates when localPatientId:" + localPatientId + " from systemId:" + f.getSystemId());
                } catch (Exception e) {
                    log.warn("exception during unsubscribe localPatientId:" + localPatientId + " from systemId:" + f.getSystemId(), e);
                }

            }

        }
    }

    public IPatientDAO getPatientDao() {
        return patientDao;
    }

    public void setPatientDao(IPatientDAO patientDao) {
        this.patientDao = patientDao;
    }

    public ISolrDao getSolrDao() {
        return solrDao;
    }

    public void setSolrDao(ISolrDao solrDao) {
        this.solrDao = solrDao;
    }

    public IVistaVprDataExtractEventStreamDAO getVistaPatientDataService() {
        return vistaPatientDataService;
    }

    public void setVistaPatientDataService(IVistaVprDataExtractEventStreamDAO vistaPatientDataService) {
        this.vistaPatientDataService = vistaPatientDataService;
    }

    public IVistaOperationalDataDAO getVistaOperationalDataService() {
        return vistaOperationalDataService;
    }

    public void setVistaOperationalDataService(IVistaOperationalDataDAO vistaOperationalDataService) {
        this.vistaOperationalDataService = vistaOperationalDataService;
    }

    public ISyncService getSyncService() {
        return syncService;
    }

    public void setSyncService(ISyncService syncService) {
        this.syncService = syncService;
    }
}
