package gov.va.hmp;

import org.springframework.core.env.Environment;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.*;

public class HmpProperties {
    public static final String HMP_HOME_ENVIRONMENT_VARIABLE_NAME = "HMP_HOME";
    public static final String HMP_HOME_SYSTEM_PROPERTY_NAME = "hmp.home";
    public static final String HMP_PROPERTIES_FILE_NAME = "hmp.properties";

    public static final String DEVELOPMENT_PROFILE = "development";

    public static final String VERSION = "hmp.version";
    public static final String BUILD_DATE = "hmp.build.date";
    public static final String BUILD_NUMBER = "hmp.build.number";

    public static final String SETUP_COMPLETE = "hmp.setup.complete";
    public static final String SERVER_ID = "hmp.server.id";
    public static final String PROPERTIES_ENCRYPTED = "hmp.properties.encrypted";
    public static final String SERVER_HOST = "hmp.server.host";
    public static final String SERVER_PORT_HTTP = "hmp.server.port.http";
    public static final String SERVER_PORT_HTTPS = "hmp.server.port.https";
    public static final String SERVER_URL = "hmp.server.url";
    public static final String HMP_DATA_DIR = "hmp.data.dir";

    public static final String JDS_URL = "jds.url";
    public static final String JDS_CONN_MANAGER_MAX_PER_ROUTE = "jds.conn-manager.max-per-route";

    public static final String ACTIVEMQ_BROKER_URL = "activemq.broker.url";
    public static final String ACTIVEMQ_DATA_DIR = "activemq.data.dir";

    public static final String EHCACHE_DATA_DIR = "ehcache.disk.store.dir";

    public static final String SOLR_URL = "solr.url";
    public static final String SOLR_CONN_MANAGER_MAX_PER_ROUTE = "solr.conn-manager.max-per-route";
    public static final String SOLR_UPDATE_QUEUE_SIZE = "solr.update.queueSize";
    public static final String SOLR_UPDATE_THREAD_COUNT = "solr.update.threadCount";
    public static final String SOLR_REINDEX_ON_STARTUP = "solr.autoReindexOnStartupEnabled";    
    public static final String SOLR_REINDEX_BATCH_SIZE = "solr.reindexBatchSize";

    public static final String INFO_BUTTON_URL = "openinfobutton.url";

    public static final String JIRA_USERNAME = "jira.username";
    public static final String JIRA_PASSWORD = "jira.password";
    public static final String JIRA_HOST = "jira.host";
    public static final String JIRA_PROJECT = "jira.project";

    public static final String SYNC_POLL_INITIAL_DELAY_SECONDS = "sync.poll.initialDelay.seconds";
    public static final String SYNC_POLL_PERIOD_SECONDS = "sync.poll.period.seconds";
    public static final String SYNC_BATCH_SIZE = "sync.batch.size";
    public static final String SYNC_MAX_SIZE_BYTES = "sync.max.size.bytes";
    public static final String SYNC_TARGET_TIME_SECONDS = "sync.target.time.seconds";
    public static final String SYNC_MAX_RETRY_HOURS="sync.max.retry.hours";
    public static final String SYNC_PAUSE_BEFORE_RETRY_SECONDS = "sync.pause.before.retry.seconds";
    public static final String SYNC_RETRY_ON_VISTA_ERROR_ATTEMPTS = "sync.retry.on.vista.error.attempts";
    public static final String SYNC_RPC_MIN_EVICTABLE_IDLE_TIME_SECONDS = "sync.rpc.minEvictableIdleTime.seconds";
    public static final String SYNC_RPC_MAX_ACTIVE = "sync.rpc.maxActive";
    public static final String SYNC_RPC_TIMEOUT_SECONDS = "sync.rpc.timeout.seconds";
    public static final String SYNC_INTEGRITY_CHECK_INTERVAL_SECONDS = "sync.integrity.check.interval.seconds";

    public static final String HDR_BASEURL = "hdr.base.url";
    public static final String HDR_ENABLED = "hdr.enabled";

    public static final String TASK_POOL_SIZE = "task.poolSize";
    public static final String TASK_SYNC_ON_INIT_DELAY_SECONDS = "task.syncOnApplicationInit.initialDelay.seconds";
    public static final String TASK_JDS_HTTP_CONNECTION_EVICTION_PERIOD_SECONDS = "task.jdsHttpConnectionEvictor.period.seconds";
    public static final String TASK_REFRESH_ASU_POLICIES_INITIAL_DELAY_SECONDS = "task.refreshAsuPolicies.initialDelay.seconds";
    public static final String TASK_REFRESH_ASU_POLICIES_PERIOD_SECONDS = "task.refreshAsuPolicies.period.seconds";

    public static Map<String, String> getProperties(Environment environment) {
        return getProperties(environment, false);
    }

    public static Map<String, String> getProperties(Environment environment, boolean includeSensitive) {
        Map<String, String> props = new HashMap<String, String>();
        for (String prop : HmpProperties.getPropertyNames(includeSensitive)) {
            props.put(prop, environment.getProperty(prop));
        }
        return Collections.unmodifiableMap(props);
    }

    public static Set<String> getPropertyNames() {
        return getPropertyNames(false);
    }

    public static Set<String> getPropertyNames(boolean includeSensitive) {
        HashSet<String> s = new HashSet<String>();
        Field[] fields = HmpProperties.class.getFields();
        for (Field f : fields) {
            if (Modifier.isPublic(f.getModifiers()) && Modifier.isStatic(f.getModifiers()) && Modifier.isFinal(f.getModifiers()) && String.class.isAssignableFrom(f.getType())) {
                try {
                    String propertyName = f.get(null).toString();
                    if (isSensitive(propertyName)) {
                        if (includeSensitive) s.add(propertyName);
                    } else {
                        s.add(propertyName);
                    }
                } catch (IllegalAccessException e) {
                    // NOOP: shouldn't ever happen because we're only examining public fields
                }
            }
        }
        return Collections.unmodifiableSet(s);
    }


    private static boolean isSensitive(String propertyName) {
        String[] disallowed = new String[]{"password", "username", "credent", "passwd", "usrname"};
        for (String s : disallowed) {
            if (propertyName.toLowerCase().indexOf(s) >= 0) {
                return true;
            }
        }
        return false;
    }
}
