package gov.va.hmp.access.asu;

import gov.va.cpe.vpr.DocumentDefinition;
import gov.va.cpe.vpr.frameeng.*;
import gov.va.cpe.vpr.pom.POMObjectEvent;
import gov.va.cpe.vpr.queryeng.MergedDocumentsViewDef;
import gov.va.cpe.vpr.queryeng.ViewDef;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

@Component
public class RefreshAsuPolicies extends Frame implements Runnable {

    private AsuPolicyDecisionPoint pdp;
    private IFrameRunner runner;
    private ApplicationContext ctx;

    @Autowired
    public void setFrameRunner(IFrameRunner runner) {
        this.runner = runner;
    }

    @Autowired(required=false)
    public void setPolicyDecisionPoint(AsuPolicyDecisionPoint pdp) {
        this.pdp = pdp;
    }

    public void setApplicationContext(ApplicationContext ctx) {
        this.ctx = ctx;
    }

    @Override
    public void run() {
        doRefresh();
    }

    private void doRefresh() {
        if (this.pdp != null) {
            this.pdp.refresh();
        }
    }

    @Override
    public void exec(FrameJob.FrameTask task) throws FrameException {
        boolean doIt = task.getTriggerEvent() != null && (task.getTriggerEvent() instanceof POMObjectEvent || task.getTriggerEvent() instanceof IFrameEvent.OperationalSyncCompleteEvent);
        if (doIt) {
            doRefresh();
            ViewDef docsViewDef = ctx.getBean(MergedDocumentsViewDef.class);
            if(docsViewDef!=null) {
                // super-hacky-lame-sauce, I know, but without being able to ensure a sequence of event triggers, I can't do this the clean way (put triggers in the MergedDocsViewDef to refresh them) because they might refresh before the ASU refresh has finished.
                task.addAction(new ViewDef.ViewDefRefreshAction(docsViewDef, (POMObjectEvent<?>) task.getTriggerEvent()));
            }
        }
    }

    @Override
    protected void doInit(FrameJob task) {
        addTrigger(new POMObjectEventTrigger<>(AsuRuleDef.class));
        addTrigger(new POMObjectEventTrigger<>(DocumentDefinition.class));
        addTrigger(new IFrameTrigger.OperationalSyncCompleteTrigger());
    }
}
