package gov.va.hmp.metrics.health;

import com.codahale.metrics.health.HealthCheck;
import com.codahale.metrics.health.HealthCheckRegistry;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.config.BeanPostProcessor;

public class HealthCheckBeanPostProcessor implements BeanPostProcessor {

    private static final Logger LOG = LoggerFactory.getLogger(HealthCheckBeanPostProcessor.class);

    private final HealthCheckRegistry healthChecks;

    public HealthCheckBeanPostProcessor(final HealthCheckRegistry healthChecks) {
        this.healthChecks = healthChecks;
    }

    @Override
    public Object postProcessBeforeInitialization(Object bean, String beanName) {
        return bean;
    }

    @Override
    public Object postProcessAfterInitialization(Object bean, String beanName) {
        if (bean instanceof HealthCheck) {
            healthChecks.register(beanName, (HealthCheck) bean);

            LOG.debug("Registering HealthCheck bean {}", beanName);
        }

        return bean;
    }
}
