package gov.va.hmp.spring;

import gov.va.cpe.vpr.pom.jds.JdsHttpConnectionEvictor;
import gov.va.cpe.vpr.sync.SyncIntegrityCheck;
import gov.va.cpe.vpr.sync.SyncOnApplicationInit;
import gov.va.cpe.vpr.sync.vista.VprUpdateJob;
import gov.va.cpe.vpr.sync.vista.VprUpdateTrigger;
import gov.va.hmp.HmpProperties;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.scheduling.TaskScheduler;
import org.springframework.scheduling.Trigger;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;

import java.util.Date;

import static gov.va.hmp.HmpProperties.*;
import static java.util.concurrent.TimeUnit.SECONDS;

@Configuration
public class SchedulingConfig {

    @Autowired
    Environment env;

    @Autowired
    private Runnable vprUpdateJob;

    @Autowired
    private Runnable syncOnApplicationInit;

    @Autowired
    private Runnable jdsHttpConnectionEvictor;

    @Autowired
    private Runnable syncIntegrityCheck;

    @Bean
    public TaskScheduler taskScheduler() {
        ThreadPoolTaskScheduler taskScheduler = new ThreadPoolTaskScheduler();
        taskScheduler.setDaemon(true);
        taskScheduler.setPoolSize(env.getRequiredProperty(TASK_POOL_SIZE, Integer.class));
        taskScheduler.afterPropertiesSet();

        taskScheduler.schedule(vprUpdateJob, vprUpdateTrigger());
        taskScheduler.schedule(syncOnApplicationInit, new Date(System.currentTimeMillis() + SECONDS.toMillis(env.getRequiredProperty(TASK_SYNC_ON_INIT_DELAY_SECONDS, Integer.class))));
        taskScheduler.scheduleWithFixedDelay(jdsHttpConnectionEvictor, SECONDS.toMillis(env.getRequiredProperty(TASK_JDS_HTTP_CONNECTION_EVICTION_PERIOD_SECONDS, Integer.class)));
        taskScheduler.scheduleWithFixedDelay(syncIntegrityCheck, SECONDS.toMillis(env.getRequiredProperty(SYNC_INTEGRITY_CHECK_INTERVAL_SECONDS, Integer.class)));

        return taskScheduler;
    }

    @Bean
    public Trigger vprUpdateTrigger() {
        VprUpdateTrigger vprUpdateTrigger = new VprUpdateTrigger(env.getRequiredProperty(HmpProperties.SYNC_POLL_PERIOD_SECONDS, Integer.class), SECONDS);
        vprUpdateTrigger.setInitialDelay(env.getRequiredProperty(HmpProperties.SYNC_POLL_INITIAL_DELAY_SECONDS, Integer.class));
        return vprUpdateTrigger;
    }
}
