Ext.define('gov.va.cpe.patient.CWADFButton', {
    alias: 'widget.cwadfbutton',
    extend: 'gov.va.hmp.ux.PopUpButton',
    ui: 'transparent',
    maxWidth: 240,
    mixins: {
        patientaware: 'gov.va.hmp.PatientAware'
    },
    cls: 'hmp-pt-bar-item',
    tpl: new Ext.XTemplate(
        '<div class="cpe-cwadf">' +
            '<tpl if="cwadf">' +
            '<div class="cwadf-row">',
        '<tpl if="this.hasCrisisNotes(cwadf)"><span class="label label-danger">Crisis&nbsp;Notes</span></tpl>',
        '<tpl if="this.hasWarnings(cwadf)"><span class="label label-danger">Warning&nbsp;Notes</span></tpl>',
        '</div>',
        '<div class="cwadf-row">',
        '<tpl if="this.hasAllergies(cwadf)"><span class="label label-danger">Allergies</span></tpl>',
        '<tpl if="this.hasFlags(cwadf)"><span class="label label-danger">Flags</span></tpl>',
        '<tpl if="this.hasDirectives(cwadf)"><span class="label label-danger">Directives</span></tpl>',
            '</div>' +
            '</tpl>',
        '<tpl if="!cwadf">' +
            '<div class="text-muted" style="text-align: center">Postings</div>',
        '<h3 style="text-align: center">None</h3>',
            '</tpl>' +
        '</div>',
        {
            disableFormats: true,
            hasCrisisNotes: function(cwadf) {
                return cwadf.indexOf('C') != -1;
            },
            hasWarnings: function(cwadf) {
                return cwadf.indexOf('W') != -1;
            },
            hasAllergies: function(cwadf) {
                return cwadf.indexOf('A') != -1;
            },
            hasDirectives: function(cwadf) {
                return cwadf.indexOf('D') != -1;
            },
            hasFlags: function(cwadf) {
                return cwadf.indexOf('F') != -1;
            }
        }),
    menuAlign:'tr-br?',
    menuOffsets:[0,0],
    popUp:{
        xtype: 'component',
        id: 'cwadf-detail',
        autoEl: {
            tag: 'div',
            'data-spy': 'scroll',
            'data-target': '#cwadf-nav'
        },
        width: 834,
        height: 600,
        overflowY: 'scroll',
        loader: {
            autoLoad: false
        }
    },

    // @private
    initComponent: function() {
        var me = this;

        // create/configure the store for this grid
        me.store = Ext.create('Ext.data.Store', {
            autoLoad: false,
            proxy: {
                //simpleSortMode: true,
                type: 'ajax',
                url: '/vpr/view/gov.va.cpe.vpr.queryeng.CWADFViewDef',
                reader: {
                    type: 'json',
                    root: 'data',
                    totalProperty: 'totalItems'
                },
                limitParam: 'row.count',
                startParam: 'row.start',
                groupParam: 'group',
                sortParam: 'sort',
                directionParam: 'sort.dir',
                timeout: 30000,
                // will be updated by setViewDef()
                extraParams: {
                    mode: 'json'
                }
            }
        });

        // initalize component (and detail view)
        this.callParent(arguments);

        // attach listeners
        this.on('patientchange', this.patientchange, this);
        this.store.on('load', this.onLoad, this);
//		// handlers for exceptions
        this.store.getProxy().on('exception', function(proxy, response, operation, eOpts) {
            // clear the store, hide the loading mask, handle the error
            me.store.removeAll();
            if(response.status!=0) {
                var error = gov.va.hmp.ErrorHandler.handle(response);

                if (me.rendered && !me.hidden) {
                    me.setLoading({
                        msg: '<h4>Oops</h4><p>' + error.msg + '</p>' +
                        '<table class="hmp-labeled-values">' +
                        '<tr><td>status</td><td>'+error.statusLine+'</td></tr>' +
                        '<tr><td>url</td><td>'+error.url+'</td></tr>' +
                        '<tr><td>error</td><td>'+error.detail.error+'</td></tr>' +
                        '<tr><td>method</td><td>' + error.detail.className + '.' +  error.detail.methodName + '()</td></tr>' +
                        '<tr><td>line</td><td>'+error.detail.file+':'+error.detail.line+'</td></tr>' +
//                        '<tr><td>stackTrace</td><td style="height: 100px; overflow-y: scroll">'+error.detail.stackTrace+'</td></tr>' +
                        '</table>',
                        cls: 'alert alert-danger'
                    });
                }
            }
        });
        this.store.on('beforeload', function(store, op, eopts) {
            op.skipErrors = true;
            store.removeAll();
            me.setLoading(false);
        });

        gov.va.hmp.EventBus.on('viewdefUpdate', this.onViewDefUpdate, this);
    },

    onViewDefUpdate: function(msg) {
        var viewId = msg['viewdef.name'];
        if ('gov.va.cpe.vpr.queryeng.CWADFViewDef' != viewId) return;
        this.store.reload();
    },

    // This is the target of the patientchange listener
    patientchange: function(pid) {
        this.pid = pid;
        Ext.apply(this.store.proxy.extraParams,{pid: pid});
        this.store.reload();
    },

    // this is the target of the load event on the store
    onLoad: function(store, records, successful, eOpts) {
        var me = this;
        // Manually update the button tpl
        var c, w, a, d, f, cwad="";
        for(idx in records) {
            var rec = records[idx];
            if(rec.get('uid').indexOf('allergy')>0) {
                a=true;
            }
            if(rec.get('uid').indexOf('patient')>0) {
                f=true;
            }
            if(rec.get('uid').indexOf('document')>0) {
                if(rec.get('documentTypeCode')=='C') {
                    c=true;
                }
                if(rec.get('documentTypeCode')=='D') {
                    d=true;
                }
                if(rec.get('documentTypeCode')=='W') {
                    w=true;
                }
            }
        }
        if(c) {cwad+="C";}
        if(w) {cwad+="W";}
        if(a) {cwad+="A";}
        if(d) {cwad+="D";}
        if(f) {cwad+="F";}
        this.update({cwadf:cwad});
    },

    onDestroy : function() {
        gov.va.hmp.EventBus.un('viewdefUpdate', this.onViewDefUpdate, this);
        this.callParent(arguments);
    }
});