Ext.define('gov.va.hmp.tabs.DocumentsPanel', {
    extend: 'gov.va.hmp.containers.MultiGridPanel',
    requires: [
        'gov.va.hmp.healthtime.PointInTime',
        'gov.va.cpe.viewdef.HealthTimeColumn',
        'gov.va.cpe.viewdef.FacilityColumn',
        'gov.va.cpe.patient.PatientAwareTab'
    ],
    mixins: {
        patientawaretab: 'gov.va.cpe.patient.PatientAwareTab'
    },
    alias: 'widget.documentspanel',
    title: 'Merged Docs',
    detailType: 'right',
    padding: '0 5',
    items: [
        {
            xtype: 'viewdefgridpanel',
            ui: 'underlined-title-condensed',
            flex: 2,
            viewID: 'gov.va.cpe.vpr.queryeng.MergedDocumentsViewDef',
            detailTitleTpl: '{localTitle}',
            scroll: true,
            columns: [
                { text: 'Title', dataIndex: 'summary', flex: 1, minWidth: 200, renderer: function(value) {return '<span title="'+value+'">'+value+'</span>';} },
                { text: 'Date/Time', xtype: 'healthtimecolumn', dataIndex: 'dateTime' },
                { text: 'Type', dataIndex: 'kind' },
                { text: 'Author', dataIndex: 'authorDisplayName' },
                { xtype: 'facilitycolumn' }
            ],
            storeConfig: {
                cursor: {
                    next: "",
                    previous: ""
                },
                clearOnPageLoad: false,
                onProxyLoad: function(operation) {
                    /*
                     Copied from Ext.data.Store class 4.2.2
                     */

                    var resp = Ext.decode(operation.response.responseText);
                    this.cursor.previous = resp.previous;
                    this.cursor.next = resp.next;

                    var me = this, resultSet = operation.getResultSet(), records = operation.getRecords(), successful = operation.wasSuccessful();

                    if (me.isDestroyed) {
                        return;
                    }

                    if (resultSet) {
                        // With cursor-based paging, we will be amending totals, not replacing them.
                        if(me.totalCount) {
                            me.totalCount = me.totalCount + records.length;
                        } else {
                            me.totalCount = records.length;
                        }
                    }

                    // Loading should be set to false before loading the records.
                    // loadRecords doesn't expose any hooks or events until refresh
                    // and datachanged, so by that time loading should be false
                    me.loading = false;
                    if (successful) {
                        if(operation.addRecords) {
                            me.add(records);
                        } else {
                            me.loadRecords(records, operation);
                        }
                    }

                    if (me.hasListeners.load) {
                        me.fireEvent('load', me, records, successful);
                    }

                    //TODO: deprecate this event, it should always have been 'load' instead. 'load' is now documented, 'read' is not.
                    //People are definitely using this so can't deprecate safely until 2.x
                    if (me.hasListeners.read) {
                        me.fireEvent('read', me, records, successful);
                    }

                    //this is a callback that would have been passed to the 'read' function and is optional
                    Ext.callback(operation.callback, operation.scope || me, [records, operation, successful]);
                }
            },
            viewParams: {
                'row.cursor': 0,
                'row.count': 100,
                'row.limit': 100
            }
        }
    ],

    onBoxReady: function() {
        var me = this;
        me.callParent(arguments);
        var grid = me.down('viewdefgridpanel');
        var store = grid.getStore();
        var proxy = store.getProxy();
        grid.view.getEl().on('scroll', function(e, t) {
            console.log("scroll", t.scrollTop, t.scrollHeight,grid.el.dom.clientHeight,(grid.el.dom.clientHeight + t.scrollTop - t.scrollHeight));
            var gridHeight = t.scrollHeight;
            var scrollHeight = t.scrollTop+grid.el.dom.clientHeight;
            var dY = scrollHeight-gridHeight;
            if(dY>=0) {
                console.log('bottom reached; digging for more data');
                // TODO: Load incrementally cursor paging

                if(store.cursor && store.cursor.next) {
                    proxy.extraParams['row.cursor']=store.cursor.next;
                    store.cursor.next="";
                    store.load({addRecords: true});
                }
            }
        });
        store.on('beforeload',function(ds, operation, eOpts) {
            if(!(operation.addRecords)) {
                // Reset cursor and proxy values
                store.cursor.next="";
                store.cursor.previous="";
                proxy.extraParams['row.cursor']=0;
            }
        })
    }
});