package gov.va.cpe.vpr;

import org.junit.Assert;
import org.junit.Test;

import java.util.LinkedHashMap;

public class VitalSignTests {
    @Test
    public void testSummaryWithoutInterpretation() {
        LinkedHashMap<String, Object> map = new LinkedHashMap<String, Object>(5);
        map.put("typeName", "WEIGHT");
        map.put("result", "182");
        map.put("units", "lb");
        map.put("metricResult", "82.73");
        map.put("metricUnits", "kg");
        VitalSign s = new VitalSign(map);
        Assert.assertEquals("WEIGHT 182 lb", s.getSummary());
    }

    @Test
    public void testSummaryWithInterpretation() {
        LinkedHashMap<String, Object> map = new LinkedHashMap<String, Object>(6);
        map.put("typeName", "WEIGHT");
        map.put("result", "182");
        map.put("units", "lb");
        map.put("metricResult", "82.73");
        map.put("metricUnits", "kg");
        map.put("interpretationCode", "L");
        VitalSign s = new VitalSign(map);
        Assert.assertEquals("WEIGHT 182L lb", s.getSummary());
    }

    @Test
    public void testSummaryWithoutUnits() {
        LinkedHashMap<String, Object> map = new LinkedHashMap<String, Object>(2);
        map.put("typeName", "BLOOD PRESSURE");
        map.put("result", "134/81");
        VitalSign s = new VitalSign(map);
        Assert.assertEquals("BLOOD PRESSURE 134/81", s.getSummary());
    }

    @Test
    public void testSummaryWithoutType() {
        LinkedHashMap<String, Object> map = new LinkedHashMap<String, Object>(1);
        map.put("result", "134/81");
        VitalSign s = new VitalSign(map);
        Assert.assertEquals(" 134/81", s.getSummary());
    }

    @Test
    public void testSummaryWithoutAnything() {
        VitalSign s = new VitalSign();
        Assert.assertEquals("", s.getSummary());
    }

}
