package gov.va.cpe.vpr.frameeng;

import gov.va.cpe.vpr.PatientDemographics;
import gov.va.cpe.vpr.frameeng.FrameAction.BaseFrameAction;
import gov.va.cpe.vpr.frameeng.FrameJob.FrameTask;
import gov.va.cpe.vpr.frameeng.FrameRegistry.IFrameLoader;
import gov.va.cpe.vpr.frameeng.FrameRegistry.ProtocolFileFrameLoader;
import gov.va.cpe.vpr.frameeng.FrameRegistry.StaticFrameLoader;
import gov.va.cpe.vpr.frameeng.IFrameTrigger.PatientObjectFieldChangedTrigger;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;

import java.io.File;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.List;
import java.util.Map;

import static org.junit.Assert.*;

public class FrameRegistryTests {
	
	public static class TestFrame extends Frame {
		
		public TestFrame() {
			setID("testframe");
			setName("Test Frame");
			addTrigger(new PatientObjectFieldChangedTrigger<PatientDemographics>(PatientDemographics.class, "givenName"));
		}

		@Override
		public void exec(FrameTask ctx) throws FrameException {
			ctx.addAction(new EchoAction("Hello World Action"));
		}
	}
	
	private static class EchoAction extends BaseFrameAction {
		private String str;

		public EchoAction(String str) {
			this.str = str;
		}
		
		public String getStr() {
			return str;
		}
	}

	public FrameRegistry registry = null;
	public IFrameLoader loader = null;
	public TestFrame testFrame;
	
	@Before
	public void setup() throws Exception {
		testFrame = new TestFrame();
		loader = new StaticFrameLoader(testFrame);
		registry = new FrameRegistry(loader);
	}
	
	@Test
	public void testFrameMeta() {
		Frame frame = new TestFrame();
		assertEquals("testframe", frame.getID());
		assertEquals("Test Frame", frame.getName());
		
		List<IFrameTrigger<?>> triggers = frame.getTriggers();
		assertEquals(1, triggers.size());
	}
	
	@Test
	@Ignore
	public void testProtocolFileFrameLoader() throws URISyntaxException {
		URL url = FrameRegistryTests.class.getResource("/gov/va/cpe/vpr/frames/");
		ProtocolFileFrameLoader loader = new ProtocolFileFrameLoader(new File(url.toURI()));
		List<IFrame> frames = loader.load();
		assertEquals(6, frames.size());
		IFrame frame = frames.get(0);
		Map<String, Object> meta = frame.getMeta();
		assertEquals("TEST_protocol_id", frame.getID());
		assertTrue(new File(frame.getResource()).exists());
		assertEquals("MRSA Infection", frame.getName());
		assertEquals("Acute Conditions", meta.get("conditionStatus"));
	}
	
	@Test
	@Ignore
	public void testFrameRegistry() throws Exception {
		// confirm the frame loader/registry
		assertEquals(1, registry.size());
		assertEquals(false, registry.isEmpty());
		assertNotNull(registry.findByID("testframe"));
		assertEquals(1, registry.getFrameLoaders().size());
		assertSame(loader, registry.getFrameLoaders().get(0));
	}
	
}
