package gov.va.cpe.vpr.sync.msg;

import gov.va.cpe.vpr.Allergy;
import gov.va.cpe.vpr.dao.ISolrDao;
import gov.va.cpe.vpr.pom.IGenericPatientObjectDAO;
import gov.va.hmp.HmpProperties;

import org.apache.solr.client.solrj.SolrServer;
import org.junit.Test;
import org.mockito.Mockito;
import org.springframework.core.env.Environment;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.jms.support.converter.SimpleMessageConverter;

import javax.jms.Message;
import javax.jms.Session;

import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

import static org.mockito.Mockito.*;

public class ReindexPatientMessageHandlerTest {
    @Test
    public void testOnMessage() throws Exception {
        ReindexPatientMessageHandler handler = new ReindexPatientMessageHandler();
        
        Environment env = Mockito.mock(Environment.class);
        when(env.getProperty(eq(HmpProperties.SOLR_REINDEX_BATCH_SIZE), eq(Integer.class), any(Integer.class))).thenReturn(Integer.valueOf(123));

        ISolrDao solrServiceMock = Mockito.mock(ISolrDao.class);
        Map<String, Long> solrCntMap = new HashMap<>();
        solrCntMap.put("allergy", new Long(1));
        when(solrServiceMock.findDomainCounts(eq("3"))).thenReturn(solrCntMap);

        IGenericPatientObjectDAO genericPatientRelatedDaoMock = Mockito.mock(IGenericPatientObjectDAO.class);
        Map<String, Long> jdsCntMap = new HashMap<>();
        jdsCntMap.put("allergy", new Long(2));
        when(genericPatientRelatedDaoMock.findDomainCountsByPid("3")).thenReturn(jdsCntMap);

        handler.setSolrDao(solrServiceMock);
        handler.setEnvironment(env);
        handler.setGenericPatientRelatedDao(genericPatientRelatedDaoMock);

        LinkedHashMap<String, Object> allergyValues = new LinkedHashMap<String, Object>(2);
        allergyValues.put("pid", "3");
        allergyValues.put("summary", "test");
        Allergy al = new Allergy(allergyValues);
        Page<Allergy> page = new PageImpl(Collections.singletonList(al));
        when(genericPatientRelatedDaoMock.findAllByPID(eq(Allergy.class), eq("3"), any(PageRequest.class))).thenReturn(page);

        LinkedHashMap<String, String> map1 = new LinkedHashMap<String, String>(1);
        map1.put("patientId", "3");
        Session mockSession = Mockito.mock(Session.class);
        Message mockMessage = Mockito.mock(Message.class);
        SimpleMessageConverter converter = Mockito.mock(SimpleMessageConverter.class);
        handler.converter = converter;
        when(converter.fromMessage(mockMessage)).thenReturn(map1);
        handler.onMessage(mockMessage, mockSession);
        verify(genericPatientRelatedDaoMock, times(1)).findAllByPID(eq(Allergy.class), eq("3"), any(PageRequest.class));
        verify(solrServiceMock, times(1)).deleteByQuery(eq("pid:3 AND domain:allergy"));
        verify(solrServiceMock, times(1)).save(al);
    }

}
