# VistA Support

VistA Support makes it easy to communicate and exchange data with VistA servers from
java applications, especially Spring-powered web applications.

Its primary goals are to:

* Leverage the maturity of the Delphi RPC Broker client used in CPRS by porting it to java and creating an extensive test suite
* Provide a high level, web-friendly API for calling VistA RPCs via RPC URIs
* Facilitate the implementation of alternative RPC protocols (such as VistALink or Caché object based ones) with a layer of abstraction
* Provide ease of integration with Spring and Spring Security based web applications by using standard programming models and configuration idioms
* Support edge case functionality heretofore unimplemented by non-Delphi VistA RPC clients (such as changing expired Verify Codes and authenticating with Broker Security Enhancement (BSE) or CCOW tokens)

## Building from Source
The VistA Support libraries use a [Gradle][]-based build system. In the instructions
below, [`./gradlew`][] is invoked from the root of the source tree and serves as
a cross-platform, self-contained bootstrap mechanism for the build.

### Prerequisites

[JDK 7 update 75 or later][JDK7 build]

Be sure that your `JAVA_HOME` environment variable points to the `jdk1.7.0` folder
extracted from the JDK download.

### Install all vista-\* jars into your local Maven cache
`./gradlew publishToMavenLocal`

### Compile and test; build all jars, distribution zips, and docs
`./gradlew build`

... and discover more commands with `./gradlew tasks`. See also the [Gradle User Guide][].

## Modules
There are a number of modules in the VistA Support project, here is a quick overview:

### vista-rpc-client
The main library providing the features that support the other modules in VistA Support,
these include:

* The `RpcTemplate` class, providing static convenience methods that make it easy to call VistA RPCs from java programs.
* Support for VistA RPC URIs
* RPC and network statistics exposed via JMX
* St utilities for

### vista-auth
Provides support for using Spring Security with VistA authentication. Like the Delphi RPC Broker, it supports via
access/verify code pairs, Broker Security Enhancement (BSE) and CCOW tokens via either form-based or HTTP BASIC mechanisms.

### vista-rpc-client-metrics
Small library for exposing RPC and network statistics via [Dropwizard Metrics][]

### vista-rpc-connection-test
The VistA RPC Connection Test application provides an executable jar that makes it super easy to test whether a
particular VistA system's RPC Broker listener is up and whether or not an access/verify code pair is valid.  Additionally
can fetch a VistA system's unique ID (defined as the CRC-16 of the DOMAIN NAME(.01) field in the KERNEL SYSTEM PARAMETERS(8989.3) file).

### vista-rpc-ws
The VistA RPC Web Service module provides a web service application that makes it possible to call RPCs via HTTP. Any
HTTP client can be used to register VistA systems and invoke RPCs via the RESTful API.

## Samples

See the README for each module for usage samples.

[Dropwizard Metrics]:https://dropwizard.github.io/metrics
[Gradle]: http://gradle.org
[`./gradlew`]: http://vimeo.com/34436402
[Gradle User Guide]: http://gradle.org/docs/current/userguide/userguide_single.html
[JDK7 build]: http://www.oracle.com/technetwork/java/javase/downloads/jdk7-downloads-1880260.html
[Spring Boot]: http://docs.spring.io/spring-boot/docs/current/reference/htmlsingle/