# VistA RPC Web Service

The VistA RPC Web Service module provides a web service application that makes it possible to call RPCs via HTTP. Any
HTTP client can be used to register VistA systems and invoke RPCs via the RESTful API.

## Running the Web Service from Source

```sh
../gradlew bootRun
```

will start the service listening at [https://localhost:9043]() by default.

## Running the Web Service from the Built Distribution

```sh
java -jar vista-rpc-ws-1.0.0.jar
```

will start the service listening at [https://localhost:9043]() by default

## Visit the Web Service API docs

API docs for the web service are built-in to the application and can be seen when you visit [https://localhost:9043]()
with a web browser.

## Customizing Server Port

Like all [Spring Boot][] applications, the port the service is listening can be set via command line, like:
```sh
java -jar vista-rpc-ws-1.0.0.jar --server.port=9000
```

or by entering `server.port=9000` in an `application.properties` file in one of the following locations:

1. A /config subdir of the current directory.
2. The current directory

This will override the default settings in the `application.properties` file that is built-in to the application.

## Customizing SSL

Similarly, if you wanted to use your own valid SSL cert instead of the built-in self-signed one, you can customize this
via [Spring Boot][] standard mechanism:
```sh
java -jar vista-rpc-ws-1.0.0.jar --server.port=9000 --server.ssl.key-store=/path/to/keystore.p12 --server.ssl.key-store-password=secret --server.ssl.key-alias=alias
```

or in an `application.properties` file with contents:
```
server.port=9000
server.ssl.key-store=/path/to/keystore.p12
server.ssl.key-store-password=secret
server.ssl.key-store-type=PKCS12
server.ssl.key-alias=selfsigned
```

## VistA Account configuration persistence

VistA account settings that are registered with the web service are by default persisted to the `~/.vista-rpc-ws/accounts`
directory of the user that is running the service.

## Other Customization

See [Spring Boot][] documentation for more information about customizing the VistA RPC Web Service application.

[Spring Boot]: http://projects.spring.io/spring-boot/