package gov.va.hmp.rpc.ws.v1;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import gov.va.hmp.vista.rpc.RpcOperations;
import gov.va.hmp.vista.rpc.RpcResponse;
import gov.va.hmp.vista.rpc.broker.protocol.RpcParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static gov.va.hmp.vista.util.RpcUriUtils.VISTA_RPC_BROKER_SCHEME;

@Controller
@RequestMapping("/v1")
public class RpcController {

    private ObjectMapper jsonMapper = new ObjectMapper();

    @Autowired
    RpcOperations rpcTemplate;

    @RequestMapping("/rpc/{vistaId}/{rpcContext}/{rpcName}")
    public
    @ResponseBody
    HttpEntity callRpc(@PathVariable String vistaId,
                       @PathVariable String rpcContext,
                       @PathVariable String rpcName,
                       @RequestParam(required = false, defaultValue = "false") Boolean suppressResponseWrapper,
                       @RequestParam MultiValueMap<String, String> httpParams) {
        List rpcParams = convertHttpToRpcParams(httpParams);
        String uri = VISTA_RPC_BROKER_SCHEME + "://" + vistaId + "/" + rpcContext + "/" + rpcName;
        RpcResponse rpcResponse = rpcTemplate.execute(uri, rpcParams);
        if (suppressResponseWrapper) {
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.TEXT_PLAIN);
            return new HttpEntity(rpcResponse.toString(), headers);
        } else {
            Map<String, Object> httpResponseWrapper = new HashMap<>();
            httpResponseWrapper.put("method", "/" + rpcContext + "/" + rpcName);
            httpResponseWrapper.put("params", rpcParams);
            httpResponseWrapper.put("result", rpcResponse.toString());
            return new HttpEntity(httpResponseWrapper);
        }
    }

    private List convertHttpToRpcParams(MultiValueMap<String, String> httpParams) {
        List rpcParams = new ArrayList();
        int i = 0;
        String paramName = "param[" + i + "]";
        while (httpParams.containsKey(paramName)) {
            List<String> httpParamValues = httpParams.get(paramName);
            if (httpParamValues.size() == 1) {
                rpcParams.add(httpParamValues.get(0));
            } else {
                rpcParams.add(httpParamValues);
            }

            i++;
            paramName = "param[" + i + "]";
        }
        for (i = 0; i < rpcParams.size(); i++) {
            Object paramValue = rpcParams.get(i);
            if (paramValue instanceof String) {
                String paramValueStr = ((String) paramValue).trim();
                if (isJSON(paramValueStr)) {
                    try {
                        JsonNode jsonNode = jsonMapper.readTree(paramValueStr);
                        rpcParams.set(i, jsonNode);
                    } catch (IOException e) {

                    }
                }
            }
        }
        return rpcParams;
    }

    // poor man's JSON detector
    private boolean isJSON(String paramValueStr) {
        String trimmed = paramValueStr.trim();
        return trimmed.startsWith("{") && trimmed.endsWith("}");
    }
}
