package gov.va.hmp.rpc.ws.v1;

import gov.va.hmp.rpc.ws.CollectionResource;
import gov.va.hmp.rpc.ws.VistaAccount;
import gov.va.hmp.rpc.ws.VistaAccountNotFoundException;
import gov.va.hmp.rpc.ws.VistaAccountRepository;
import gov.va.hmp.vista.rpc.RpcHost;
import gov.va.hmp.vista.rpc.RpcOperations;
import gov.va.hmp.vista.rpc.conn.SystemInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/v1/vista-accounts")
public class VistaAccountController {

    @Autowired
    VistaAccountRepository vistaAccountRepository;

    @Autowired
    RpcOperations rpcTemplate;

    @RequestMapping(method = RequestMethod.GET)
    public CollectionResource<VistaAccount> list(Pageable pageable) {
        Page<VistaAccount> page = vistaAccountRepository.findAll(pageable);
        return new CollectionResource(page);
    }

    @RequestMapping(method = RequestMethod.POST)
    public VistaAccount add(@RequestParam String host, @RequestParam int port) {
        SystemInfo systemInfo = rpcTemplate.fetchSystemInfo(new RpcHost(host, port));
        VistaAccount vistaAccount = new VistaAccount(systemInfo.getVistaId(), systemInfo.isProductionAccount(), host, port);
        vistaAccount = vistaAccountRepository.save(vistaAccount);
        return vistaAccount;
    }

    @RequestMapping(method = RequestMethod.DELETE)
    public long deleteAll() {
        long count = vistaAccountRepository.count();
        vistaAccountRepository.deleteAll();
        return count;
    }

    @RequestMapping(value = "/{vistaId}", method = RequestMethod.GET)
    public VistaAccount show(@PathVariable String vistaId) {
        VistaAccount account = vistaAccountRepository.findOne(vistaId);
        if (account == null) throw new VistaAccountNotFoundException(vistaId);
        return account;
    }

    @RequestMapping(value = "/{vistaId}", method = RequestMethod.DELETE)
    public VistaAccount delete(@PathVariable String vistaId) {
        VistaAccount account = vistaAccountRepository.findOne(vistaId);
        if (account == null) throw new VistaAccountNotFoundException(vistaId);
        vistaAccountRepository.delete(account);
        return account;
    }
}
