package gov.va.hmp.rpc.ws;

import org.junit.Before;
import org.junit.Test;
import org.springframework.core.MethodParameter;
import org.springframework.data.domain.Pageable;
import org.springframework.mock.web.MockHttpServletRequest;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.context.request.ServletWebRequest;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.assertThat;

public class OffsetLimitPageableHandlerMethodArgumentResolverTests {
    private MockHttpServletRequest mockRequest = new MockHttpServletRequest();
    private HandlerMethodArgumentResolver r = new OffsetLimitPageableHandlerMethodArgumentResolver();
    private MethodParameter mockControllerMethodParameter;

    @Before
    public void setUp() throws Exception {
        mockControllerMethodParameter = new MethodParameter(this.getClass().getMethod("mockControllerMethod", String.class, Pageable.class), 1);
    }

    // this here to create mock MetodParameter in @Tests
    public void mockControllerMethod(@PathVariable String id, Pageable pageable) {
        // NOOP
    }

    @Test
    public void testSupportsParameter() throws Exception {
        assertThat(r.supportsParameter(mockControllerMethodParameter), is(true));
    }

    @Test
    public void testResolveFacebookOffsetBasedPaginationParams() throws Exception {
        mockRequest.setParameter("offset", "80");
        mockRequest.setParameter("limit", "20");

        Pageable pageable = (Pageable) r.resolveArgument(mockControllerMethodParameter, null, new ServletWebRequest(mockRequest), null);

        assertThat(pageable.getPageNumber(), equalTo(4));
        assertThat(pageable.getPageSize(), equalTo(20));
    }
}
