package gov.va.hmp.rpc.ws.v1;

import gov.va.hmp.rpc.ws.CollectionResource;
import gov.va.hmp.rpc.ws.VistaAccount;
import gov.va.hmp.rpc.ws.VistaAccountRepository;
import gov.va.hmp.rpc.ws.v1.VistaAccountController;
import gov.va.hmp.vista.rpc.RpcHost;
import gov.va.hmp.vista.rpc.RpcOperations;
import gov.va.hmp.vista.rpc.conn.SystemInfo;
import org.junit.Before;
import org.junit.Test;
import org.mockito.AdditionalAnswers;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;

import java.util.Collections;

import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class VistaAccountControllerTests {

    VistaAccountController c;
    VistaAccountRepository mockVistaAccountRepostory;
    RpcOperations mockRpcTemplate;

    @Before
    public void setUp() throws Exception {
        mockVistaAccountRepostory = mock(VistaAccountRepository.class);
        mockRpcTemplate = mock(RpcOperations.class);

        c = new VistaAccountController();
        c.vistaAccountRepository = mockVistaAccountRepostory;
        c.rpcTemplate = mockRpcTemplate;
    }

    @Test
    public void testList() throws Exception {
        Pageable mockPageRequest = new PageRequest(0, 10);
        Page<VistaAccount> mockPage = new PageImpl<VistaAccount>(Collections.singletonList(new VistaAccount("ABCD", false, "example.org", 0000)), mockPageRequest, 50);
        when(mockVistaAccountRepostory.findAll(mockPageRequest)).thenReturn(mockPage);
        CollectionResource<VistaAccount> accounts = c.list(mockPageRequest);
        verify(mockVistaAccountRepostory).findAll(mockPageRequest);
    }

    @Test
    public void testDeleteAll() throws Exception {
        when(mockVistaAccountRepostory.count()).thenReturn(23L);
        long count = c.deleteAll();
        assertThat(count, is(23L));
        verify(mockVistaAccountRepostory).count();
        verify(mockVistaAccountRepostory).deleteAll();
    }

    @Test
    public void testAdd() throws Exception {
        SystemInfo mockSystemInfo = mock(SystemInfo.class);
        when(mockSystemInfo.getVistaId()).thenReturn("ABCD");
        when(mockSystemInfo.isProductionAccount()).thenReturn(false);
        
        when(mockVistaAccountRepostory.save(any(VistaAccount.class))).thenAnswer(AdditionalAnswers.returnsFirstArg());
        
        when(mockRpcTemplate.fetchSystemInfo(new RpcHost("example.org", 0000))).thenReturn(mockSystemInfo);

        VistaAccount v = c.add("example.org", 0000);
        assertThat(v.getId(), is(mockSystemInfo.getVistaId()));
        assertThat(v.isProduction(), is(mockSystemInfo.isProductionAccount()));

        verify(mockRpcTemplate).fetchSystemInfo(new RpcHost("example.org", 0000));
        verify(mockVistaAccountRepostory).save(v);
    }

    @Test
    public void testShow() throws Exception {
        VistaAccount expected = new VistaAccount("ABCD", false, "example.org", 0000);
        when(mockVistaAccountRepostory.findOne("ABCD")).thenReturn(expected);

        VistaAccount actual = c.show("ABCD");
        assertThat(actual, is(expected));

        verify(mockVistaAccountRepostory).findOne("ABCD");
    }

    @Test
    public void testDelete() throws Exception {
        VistaAccount expected = new VistaAccount("ABCD", false, "example.org", 0000);
        when(mockVistaAccountRepostory.findOne("ABCD")).thenReturn(expected);

        VistaAccount actual = c.delete("ABCD");
        assertThat(actual, is(expected));

        verify(mockVistaAccountRepostory).findOne("ABCD");
        verify(mockVistaAccountRepostory).delete(expected);
    }
}
